#! /usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("dotenv").config();
const fs = require("fs");
const commander_1 = require("commander");
const anchor = __importStar(require("@project-serum/anchor"));
const anchor_1 = require("@project-serum/anchor");
const nodewallet_1 = __importDefault(require("@project-serum/anchor/dist/cjs/nodewallet"));
const web3_js_1 = require("@solana/web3.js");
const index_1 = require("./index");
const propeasy_program_json_1 = __importDefault(require("./artifacts/propeasy_program.json"));
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const spl_token_1 = require("@solana/spl-token");
const bn_js_1 = require("bn.js");
const lodash_1 = require("lodash");
const __path = process.cwd();
const figlet = require("figlet");
const program = new commander_1.Command();
const MAINNET_RPC = "https://api-mainnet-beta.renec.foundation:8899/";
const TESTNET_RPC = "https://api-testnet.renec.foundation:8899/";
const LOCAL_RPC = "http://localhost:8899/";
console.log(figlet.textSync("Propeasy Program"));
console.log("");
const ownerKey = JSON.parse(fs.readFileSync(__path + "/sdk/src/.wallets/owner.json"));
function connectCluster(rpc, ownerKey, programId) {
    return __awaiter(this, void 0, void 0, function* () {
        const connection = new web3_js_1.Connection(rpc);
        const wallet = new nodewallet_1.default((0, index_1.keypairFromJson)(ownerKey));
        const anchorProvider = new anchor.AnchorProvider(connection, wallet, anchor.AnchorProvider.defaultOptions());
        const propeasyProgram = new anchor_1.Program(propeasy_program_json_1.default, programId, anchorProvider);
        const program = propeasyProgram;
        return program;
    });
}
// yarn cli initialize -n mainnet
program
    .command("initialize")
    .description("initialize propeasy platform")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network } = params;
    console.log("Initializing platform");
    console.log({ params });
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    if (!process.env.PROP_TOKEN) {
        console.log("PROP_TOKEN is not found in .env file");
        return;
    }
    if (!process.env.REUSD_TOKEN) {
        console.log("REUSD_TOKEN is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const pda = new index_1.PDA(programId);
    const platform = pda.platform();
    const signature = yield program.methods
        .initializePlatform(platform.bump)
        .accounts({
        admin: wallet.publicKey,
        platformMintAccount: anchor.translateAddress(process.env.PROP_TOKEN),
        purchaseMintAccount: anchor.translateAddress(process.env.REUSD_TOKEN),
        platformState: platform.key,
        systemProgram: anchor.web3.SystemProgram.programId,
        rent: web3_js_1.SYSVAR_RENT_PUBKEY,
    })
        .rpc();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
program
    .command("changeMintPlatform")
    .description("Change mint tokens of platform")
    .option("--purchaseMint <string>", "Purchase Mint Addresss")
    .option("--platformMint <string>", "Platform Mint Addresss")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { purchaseMint, platformMint, network } = params;
    console.log("Change Mint Platform");
    console.log({ params });
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    if (!purchaseMint) {
        console.log("--purchaseMint is required");
        return;
    }
    if (!platformMint) {
        console.log("--platformMint is required");
        return;
    }
    const _purchaseMint = new web3_js_1.PublicKey(purchaseMint);
    const _platformMint = new web3_js_1.PublicKey(platformMint);
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const pda = new index_1.PDA(programId);
    const platform = pda.platform();
    const signature = yield program.methods
        .changeMintTokenPlatform()
        .accounts({
        admin: wallet.publicKey,
        platformMintAccount: _platformMint,
        purchaseMintAccount: _purchaseMint,
        platformState: platform.key,
        systemProgram: anchor.web3.SystemProgram.programId,
        rent: web3_js_1.SYSVAR_RENT_PUBKEY,
    })
        .rpc();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
// yarn cli createProperty -n mainnet -pcp /sdk/src/data/default.json
program
    .command("createProperty")
    .description("create new property with json data")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-pcp, --propertyConfigPath <string>", "Property config path", "/sdk/src/data/default.json")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, propertyConfigPath } = params;
    const propertyData = JSON.parse(fs.readFileSync(__path + propertyConfigPath));
    let { tokenMint: propertyToken, privateSale: { startTime: privateSaleStartTime, endTime: privateSaleEndTime, totalSaleInToken: privateSaleSupplyAmount, priceInUsdPerToken: privateSaleTokenPrice, minPurchaseInUsd: privateSaleMinimumPurchaseAmount, commissionInPropPerUsdPurchase: privateSaleCommissionAmount, discountInPercentage: privateSaleDiscountPercentage, tgeInPercentage: privateSaleTgePercentage, unlockPeriodInSecond: privateSaleUnlockPeriod, unlockDurationInSecond: privateSaleUnlockDuration, holdRequiredInProp: privateSaleMinimumPlatformAmount, }, publicSale: { startTime: publicSaleStartTime, endTime: publicSaleEndTime, totalSaleInToken: publicSaleSupplyAmount, priceInUsdPerToken: publicSaleTokenPrice, minPurchaseInUsd: publicSaleMinimumPurchaseAmount, commissionInPropPerUsdPurchase: publicSaleCommissionAmount, boost: { purchaseAmountRequiredInUsd: publicSaleReferralCommissionBoostThreshold, commissionInPropPerUsdPurchase: publicSaleCommissionBoostAmount, }, }, } = propertyData;
    let privateSaleReferralCommissionAmount = propertyData.privateSale.referralCommission.buyerInPropPerUsdPurchase +
        propertyData.privateSale.referralCommission.referrerInPropPerUsdPurchase;
    let publicSaleReferralCommissionAmount = propertyData.publicSale.referralCommission.buyerInPropPerUsdPurchase +
        propertyData.publicSale.referralCommission.referrerInPropPerUsdPurchase;
    let publicSaleReferralCommissionBoostAmount = propertyData.publicSale.boost.referralCommission.buyerInPropPerUsdPurchase +
        propertyData.publicSale.boost.referralCommission.referrerInPropPerUsdPurchase;
    console.log("Creating new property");
    console.log({ params });
    const unixPrivateSaleStartTime = new Date(privateSaleStartTime).getTime() / 1000;
    if (isNaN(unixPrivateSaleStartTime)) {
        console.log("Error: privateSaleStartTime must be valid number");
        return;
    }
    const unixPrivateSaleEndTime = new Date(privateSaleEndTime).getTime() / 1000;
    if (isNaN(unixPrivateSaleEndTime)) {
        console.log("Error: privateSaleEndTime must be valid number");
        return;
    }
    if (isNaN(privateSaleSupplyAmount)) {
        console.log("Error: privateSaleSupplyAmount must be valid number");
        return;
    }
    if (isNaN(privateSaleTokenPrice)) {
        console.log("Error: privateSaleTokenPrice must be valid number");
        return;
    }
    if (isNaN(privateSaleMinimumPurchaseAmount)) {
        console.log("Error: privateSaleMinimumPurchaseAmount must be valid number");
        return;
    }
    if (isNaN(privateSaleReferralCommissionAmount)) {
        console.log("Error: privateSaleReferralCommissionAmount must be valid number");
        return;
    }
    if (isNaN(privateSaleDiscountPercentage)) {
        console.log("Error: privateSaleDiscountPercentage must be valid number");
        return;
    }
    if (isNaN(privateSaleTgePercentage)) {
        console.log("Error: privateSaleTgePercentage must be valid number");
        return;
    }
    if (isNaN(privateSaleUnlockPeriod)) {
        console.log("Error: privateSaleUnlockPeriod must be valid number");
        return;
    }
    if (isNaN(privateSaleUnlockDuration)) {
        console.log("Error: privateSaleUnlockDuration must be valid number");
        return;
    }
    if (isNaN(privateSaleMinimumPlatformAmount)) {
        console.log("Error: privateSaleMinimumPlatformAmount must be valid number");
        return;
    }
    const unixPublicSaleStartTime = new Date(publicSaleStartTime).getTime() / 1000;
    if (isNaN(unixPublicSaleStartTime)) {
        console.log("Error: publicSaleStartTime must be valid number");
        return;
    }
    const unixPublicSaleEndTime = new Date(publicSaleEndTime).getTime() / 1000;
    if (isNaN(unixPublicSaleEndTime)) {
        console.log("Error: publicSaleEndTime must be valid number");
        return;
    }
    if (isNaN(publicSaleSupplyAmount)) {
        console.log("Error: publicSaleSupplyAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleTokenPrice)) {
        console.log("Error: publicSaleTokenPrice must be valid number");
        return;
    }
    if (isNaN(publicSaleMinimumPurchaseAmount)) {
        console.log("Error: publicSaleMinimumPurchaseAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleReferralCommissionAmount)) {
        console.log("Error: publicSaleReferralCommissionAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleReferralCommissionBoostAmount)) {
        console.log("Error: publicSaleReferralCommissionBoostAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleReferralCommissionBoostThreshold)) {
        console.log("Error: publicSaleReferralCommissionBoostThreshold must be valid number");
        return;
    }
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const ctx = index_1.Context.withProvider(provider, program.programId);
    const pda = new index_1.PDA(programId);
    const platform = pda.platform();
    const platformData = yield ctx.fetcher.getPlatform(platform.key, true);
    if (!platformData) {
        throw new Error(`1, Platform not found`);
    }
    const purchaseMintAccount = platformData.purchaseMintAccount;
    const purchaseMintAccountInfo = yield ctx.connection.getParsedAccountInfo(purchaseMintAccount);
    if (purchaseMintAccountInfo.value == null) {
        throw new Error(`2, Info of ${purchaseMintAccount} not found`);
    }
    const purchaseDecimals = purchaseMintAccountInfo.value.data.parsed.info
        .decimals;
    const platformMintAccount = platformData.platformMintAccount;
    const platformMintAccountInfo = yield ctx.connection.getParsedAccountInfo(platformMintAccount);
    if (platformMintAccountInfo.value == null) {
        throw new Error(`3, Info of ${platformMintAccount} not found`);
    }
    const platformDecimals = platformMintAccountInfo.value.data.parsed.info
        .decimals;
    const propertyMintAccount = new web3_js_1.PublicKey(propertyToken);
    const propertyMintAccountInfo = yield ctx.connection.getParsedAccountInfo(propertyMintAccount);
    if (propertyMintAccountInfo.value == null) {
        throw new Error(`4, Info of ${propertyMintAccount} not found`);
    }
    const propertyDecimals = propertyMintAccountInfo.value.data.parsed.info
        .decimals;
    const tx = yield index_1.PropertyClient.new(ctx, wallet.publicKey, propertyMintAccount, new anchor.BN(unixPrivateSaleStartTime), new anchor.BN(unixPrivateSaleEndTime), new anchor.BN(new bignumber_js_1.default(privateSaleSupplyAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(propertyDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(privateSaleTokenPrice)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(privateSaleMinimumPurchaseAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(privateSaleCommissionAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(privateSaleReferralCommissionAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), Math.round(privateSaleDiscountPercentage * 100), Math.round(privateSaleTgePercentage * 100), new anchor.BN(privateSaleUnlockPeriod), new anchor.BN(privateSaleUnlockDuration), new anchor.BN(new bignumber_js_1.default(privateSaleMinimumPlatformAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(unixPublicSaleStartTime), new anchor.BN(unixPublicSaleEndTime), new anchor.BN(new bignumber_js_1.default(publicSaleSupplyAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(propertyDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleTokenPrice)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleMinimumPurchaseAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleCommissionAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleCommissionBoostAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleReferralCommissionAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleReferralCommissionBoostAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleReferralCommissionBoostThreshold)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()));
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
// yarn cli fundToken -n mainnet -pt {} -ft {} -ua 0
program
    .command("fundToken")
    .description("fund token to vault of property")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-pt, --propertyToken <string>", "Property token")
    .option("-ft, --fundToken <string>", "Fund token")
    .option("-ua, --uiAmount <number>", "Amount")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, fundToken, propertyToken, uiAmount } = params;
    console.log("Fund property token to vault");
    console.log({ params });
    if (isNaN(uiAmount)) {
        console.log("Error: uiAmount must be valid number");
        return;
    }
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const ctx = index_1.Context.withProvider(provider, program.programId);
    const pda = new index_1.PDA(programId);
    const mintTokenAccount = new web3_js_1.PublicKey(fundToken);
    const mintAccountInfo = yield ctx.connection.getParsedAccountInfo(mintTokenAccount);
    if (mintAccountInfo.value == null) {
        throw new Error(`5, Info of ${mintTokenAccount} not found`);
    }
    const mintDecimals = mintAccountInfo.value.data.parsed.info.decimals;
    const property = pda.property(new web3_js_1.PublicKey(propertyToken)).key;
    const payerAssociatedTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mintTokenAccount, wallet.publicKey, true);
    const propertyAssociatedTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mintTokenAccount, property, true);
    let account = yield ctx.connection.getAccountInfo(propertyAssociatedTokenAccount, "confirmed");
    const tx = new web3_js_1.Transaction();
    if (account == null) {
        tx.add(spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mintTokenAccount, propertyAssociatedTokenAccount, property, wallet.publicKey));
    }
    const amount = new spl_token_1.u64(uiAmount).mul(new bn_js_1.BN(10).pow(new bn_js_1.BN(mintDecimals)));
    tx.add(spl_token_1.Token.createTransferInstruction(spl_token_1.TOKEN_PROGRAM_ID, payerAssociatedTokenAccount, propertyAssociatedTokenAccount, wallet.publicKey, [], new spl_token_1.u64(amount.toString())));
    const txSig = yield ctx.connection.sendTransaction(tx, [wallet]);
    console.log({ txSig });
}));
// yarn cli fundToken -n mainnet -ft {} -ua 0
program
    .command("fundToken")
    .description("fund token to vault of property")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-ft, --fundToken <string>", "Fund token")
    .option("-ua, --uiAmount <number>", "Amount")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, fundToken, uiAmount } = params;
    console.log("Fund property token to vault");
    console.log({ params });
    if (isNaN(uiAmount)) {
        console.log("Error: uiAmount must be valid number");
        return;
    }
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const ctx = index_1.Context.withProvider(provider, program.programId);
    const pda = new index_1.PDA(programId);
    const platform = pda.platform();
    const platformData = yield ctx.fetcher.getPlatform(platform.key, true);
    if (!platformData) {
        throw new Error(`6, Platform not found`);
    }
    const mintTokenAccount = new web3_js_1.PublicKey(fundToken);
    const mintAccountInfo = yield ctx.connection.getParsedAccountInfo(mintTokenAccount);
    if (mintAccountInfo.value == null) {
        throw new Error(`5, Info of ${mintTokenAccount} not found`);
    }
    const mintDecimals = mintAccountInfo.value.data.parsed.info.decimals;
    const payerAssociatedTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mintTokenAccount, wallet.publicKey, true);
    const platformAssociatedTokenAccount = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mintTokenAccount, platform.key, true);
    let account = yield ctx.connection.getAccountInfo(platformAssociatedTokenAccount, "confirmed");
    const tx = new web3_js_1.Transaction();
    if (account == null) {
        tx.add(spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mintTokenAccount, platformAssociatedTokenAccount, platform.key, wallet.publicKey));
    }
    const amount = new spl_token_1.u64(uiAmount).mul(new bn_js_1.BN(10).pow(new bn_js_1.BN(mintDecimals)));
    tx.add(spl_token_1.Token.createTransferInstruction(spl_token_1.TOKEN_PROGRAM_ID, payerAssociatedTokenAccount, platformAssociatedTokenAccount, wallet.publicKey, [], new spl_token_1.u64(amount.toString())));
    const txSig = yield ctx.connection.sendTransaction(tx, [wallet]);
    console.log({ txSig });
}));
// yarn cli updateProperty -n mainnet -pcp /sdk/src/data/default.json
program
    .command("updateProperty")
    .description("update property with json data")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-pcp, --propertyConfigPath <string>", "Property config path", "/sdk/src/data/default.json")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, propertyConfigPath } = params;
    const propertyData = JSON.parse(fs.readFileSync(__path + propertyConfigPath));
    let { tokenMint: propertyToken, privateSale: { startTime: privateSaleStartTime, endTime: privateSaleEndTime, totalSaleInToken: privateSaleSupplyAmount, priceInUsdPerToken: privateSaleTokenPrice, minPurchaseInUsd: privateSaleMinimumPurchaseAmount, commissionInPropPerUsdPurchase: privateSaleCommissionAmount, discountInPercentage: privateSaleDiscountPercentage, tgeInPercentage: privateSaleTgePercentage, unlockPeriodInSecond: privateSaleUnlockPeriod, unlockDurationInSecond: privateSaleUnlockDuration, holdRequiredInProp: privateSaleMinimumPlatformAmount, }, publicSale: { startTime: publicSaleStartTime, endTime: publicSaleEndTime, totalSaleInToken: publicSaleSupplyAmount, priceInUsdPerToken: publicSaleTokenPrice, minPurchaseInUsd: publicSaleMinimumPurchaseAmount, commissionInPropPerUsdPurchase: publicSaleCommissionAmount, boost: { purchaseAmountRequiredInUsd: publicSaleReferralCommissionBoostThreshold, commissionInPropPerUsdPurchase: publicSaleCommissionBoostAmount, }, }, } = propertyData;
    let privateSaleReferralCommissionAmount = propertyData.privateSale.referralCommission.buyerInPropPerUsdPurchase +
        propertyData.privateSale.referralCommission.referrerInPropPerUsdPurchase;
    let publicSaleReferralCommissionAmount = propertyData.publicSale.referralCommission.buyerInPropPerUsdPurchase +
        propertyData.publicSale.referralCommission.referrerInPropPerUsdPurchase;
    let publicSaleReferralCommissionBoostAmount = propertyData.publicSale.boost.referralCommission.buyerInPropPerUsdPurchase +
        propertyData.publicSale.boost.referralCommission.referrerInPropPerUsdPurchase;
    console.log("Updating property");
    console.log({ params });
    const unixPrivateSaleStartTime = new Date(privateSaleStartTime).getTime() / 1000;
    if (isNaN(unixPrivateSaleStartTime)) {
        console.log("Error: privateSaleStartTime must be valid number");
        return;
    }
    const unixPrivateSaleEndTime = new Date(privateSaleEndTime).getTime() / 1000;
    if (isNaN(unixPrivateSaleEndTime)) {
        console.log("Error: privateSaleEndTime must be valid number");
        return;
    }
    if (isNaN(privateSaleSupplyAmount)) {
        console.log("Error: privateSaleSupplyAmount must be valid number");
        return;
    }
    if (isNaN(privateSaleTokenPrice)) {
        console.log("Error: privateSaleTokenPrice must be valid number");
        return;
    }
    if (isNaN(privateSaleMinimumPurchaseAmount)) {
        console.log("Error: privateSaleMinimumPurchaseAmount must be valid number");
        return;
    }
    if (isNaN(privateSaleCommissionAmount)) {
        console.log("Error: privateSaleCommissionAmount must be valid number");
        return;
    }
    if (isNaN(privateSaleReferralCommissionAmount)) {
        console.log("Error: privateSaleReferralCommissionAmount must be valid number");
        return;
    }
    if (isNaN(privateSaleDiscountPercentage)) {
        console.log("Error: privateSaleDiscountPercentage must be valid number");
        return;
    }
    if (isNaN(privateSaleTgePercentage)) {
        console.log("Error: privateSaleTgePercentage must be valid number");
        return;
    }
    if (isNaN(privateSaleUnlockPeriod)) {
        console.log("Error: privateSaleUnlockPeriod must be valid number");
        return;
    }
    if (isNaN(privateSaleUnlockDuration)) {
        console.log("Error: privateSaleUnlockDuration must be valid number");
        return;
    }
    if (isNaN(privateSaleMinimumPlatformAmount)) {
        console.log("Error: privateSaleMinimumPlatformAmount must be valid number");
        return;
    }
    const unixPublicSaleStartTime = new Date(publicSaleStartTime).getTime() / 1000;
    if (isNaN(unixPublicSaleStartTime)) {
        console.log("Error: publicSaleStartTime must be valid number");
        return;
    }
    const unixPublicSaleEndTime = new Date(publicSaleEndTime).getTime() / 1000;
    if (isNaN(unixPublicSaleEndTime)) {
        console.log("Error: publicSaleEndTime must be valid number");
        return;
    }
    if (isNaN(publicSaleSupplyAmount)) {
        console.log("Error: publicSaleSupplyAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleTokenPrice)) {
        console.log("Error: publicSaleTokenPrice must be valid number");
        return;
    }
    if (isNaN(publicSaleMinimumPurchaseAmount)) {
        console.log("Error: publicSaleMinimumPurchaseAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleCommissionAmount)) {
        console.log("Error: publicSaleCommissionAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleCommissionBoostAmount)) {
        console.log("Error: publicSaleCommissionBoostAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleReferralCommissionAmount)) {
        console.log("Error: publicSaleReferralCommissionAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleReferralCommissionBoostAmount)) {
        console.log("Error: publicSaleReferralCommissionBoostAmount must be valid number");
        return;
    }
    if (isNaN(publicSaleReferralCommissionBoostThreshold)) {
        console.log("Error: publicSaleReferralCommissionBoostThreshold must be valid number");
        return;
    }
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const ctx = index_1.Context.withProvider(provider, program.programId);
    const pda = new index_1.PDA(programId);
    const platform = pda.platform();
    const platformData = yield ctx.fetcher.getPlatform(platform.key, true);
    if (!platformData) {
        throw new Error(`6, Platform not found`);
    }
    const purchaseMintAccount = platformData.purchaseMintAccount;
    const purchaseMintAccountInfo = yield ctx.connection.getParsedAccountInfo(purchaseMintAccount);
    if (purchaseMintAccountInfo.value == null) {
        throw new Error(`7, Info of ${purchaseMintAccount} not found`);
    }
    const purchaseDecimals = purchaseMintAccountInfo.value.data.parsed.info
        .decimals;
    const platformMintAccount = platformData.platformMintAccount;
    const platformMintAccountInfo = yield ctx.connection.getParsedAccountInfo(platformMintAccount);
    if (platformMintAccountInfo.value == null) {
        throw new Error(`8, Info of ${platformMintAccount} not found`);
    }
    const platformDecimals = platformMintAccountInfo.value.data.parsed.info
        .decimals;
    const propertyMintAccount = new web3_js_1.PublicKey(propertyToken);
    const propertyMintAccountInfo = yield ctx.connection.getParsedAccountInfo(propertyMintAccount);
    if (propertyMintAccountInfo.value == null) {
        throw new Error(`9, Info of ${propertyMintAccount} not found`);
    }
    const propertyDecimals = propertyMintAccountInfo.value.data.parsed.info
        .decimals;
    const propertyClient = yield index_1.PropertyClient.getProperty(ctx, propertyMintAccount);
    const tx = yield propertyClient.update(ctx, wallet.publicKey, propertyMintAccount, new anchor.BN(unixPrivateSaleStartTime), new anchor.BN(unixPrivateSaleEndTime), new anchor.BN(new bignumber_js_1.default(privateSaleSupplyAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(propertyDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(privateSaleTokenPrice)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(privateSaleMinimumPurchaseAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(privateSaleCommissionAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(privateSaleReferralCommissionAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), Math.round(privateSaleDiscountPercentage * 100), Math.round(privateSaleTgePercentage * 100), new anchor.BN(privateSaleUnlockPeriod), new anchor.BN(privateSaleUnlockDuration), new anchor.BN(new bignumber_js_1.default(privateSaleMinimumPlatformAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(unixPublicSaleStartTime), new anchor.BN(unixPublicSaleEndTime), new anchor.BN(new bignumber_js_1.default(publicSaleSupplyAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(propertyDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleTokenPrice)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleMinimumPurchaseAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleCommissionAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleCommissionBoostAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleReferralCommissionAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleReferralCommissionBoostAmount)
        .multipliedBy(new bignumber_js_1.default(10).pow(platformDecimals))
        .toString()), new anchor.BN(new bignumber_js_1.default(publicSaleReferralCommissionBoostThreshold)
        .multipliedBy(new bignumber_js_1.default(10).pow(purchaseDecimals))
        .toString()));
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
// yarn cli withdrawToken -n mainnet -pt {} -wt {} -b {} -ua 0
program
    .command("withdrawToken")
    .description("withdraw token")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-pt, --propertyToken <string>", "Property token")
    .option("-wt, --withdrawToken <string>", "Withdraw token: {property token} or {platform token}")
    .option("-b, --beneficiary <string>", "Beneficiary")
    .option("-ua, --uiAmount <number>", "Withdraw UI amount: 0 for withdraw all", "0")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, propertyToken, withdrawToken, beneficiary, uiAmount } = params;
    console.log("Withdraw token");
    console.log({ params });
    if (isNaN(uiAmount)) {
        console.log("Error: uiAmount must be valid number");
        return;
    }
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const ctx = index_1.Context.withProvider(provider, program.programId);
    const propertyMintAccount = new web3_js_1.PublicKey(propertyToken);
    const propertyMintAccountInfo = yield ctx.connection.getParsedAccountInfo(propertyMintAccount);
    if (propertyMintAccountInfo.value == null) {
        throw new Error(`10, Info of ${propertyMintAccount} not found`);
    }
    const withdrawMintAccount = new web3_js_1.PublicKey(withdrawToken);
    const withdrawMintAccountInfo = yield ctx.connection.getParsedAccountInfo(withdrawMintAccount);
    if (withdrawMintAccountInfo.value == null) {
        throw new Error(`11, Info of ${withdrawMintAccount} not found`);
    }
    const withdrawDecimals = withdrawMintAccountInfo.value.data.parsed.info
        .decimals;
    const propertyClient = yield index_1.PropertyClient.getProperty(ctx, propertyMintAccount);
    const tx = yield propertyClient.withdrawToken(ctx, wallet.publicKey, propertyMintAccount, withdrawMintAccount, beneficiary, new anchor.BN(new bignumber_js_1.default(uiAmount).multipliedBy(new bignumber_js_1.default(10).pow(withdrawDecimals)).toString()));
    const signature = yield tx.buildAndExecute();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("DONE");
}));
// yarn cli grantBurnRole -n mainnet -u {}
program
    .command("grantBurnRole")
    .description("grant burn role")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-u, --user <string>", "Authorized user to burn token")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, user } = params;
    console.log("Grant Burn Role");
    console.log({ params });
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const pda = new index_1.PDA(programId);
    const platform = pda.platform();
    const userBurnRoleState = pda.burnRole(new web3_js_1.PublicKey(user));
    const signature = yield program.methods
        .grantBurnRole()
        .accounts({
        admin: wallet.publicKey,
        platformState: platform.key,
        userBurnRoleState: userBurnRoleState.key,
        user,
        systemProgram: anchor.web3.SystemProgram.programId,
        rent: web3_js_1.SYSVAR_RENT_PUBKEY,
    })
        .rpc();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("Granted burn role to: ", user);
}));
// yarn cli revokeBurnRole -n mainnet -u {}
program
    .command("revokeBurnRole")
    .description("revoke burn role")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-u, --user <string>", "Authorized user to burn token")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, user } = params;
    console.log("Revoke Burn Role");
    console.log({ params });
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const pda = new index_1.PDA(programId);
    const platform = pda.platform();
    const userBurnRoleState = pda.burnRole(new web3_js_1.PublicKey(user));
    const signature = yield program.methods
        .revokeBurnRole()
        .accounts({
        admin: wallet.publicKey,
        platformState: platform.key,
        userBurnRoleState: userBurnRoleState.key,
        user,
    })
        .rpc();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("Revoked burn role for: ", user);
}));
// yarn cli initializeBurn -n mainnet -t {} -ua {} -p {}
program
    .command("initializeBurn")
    .description("initialize burn info of token")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-t, --token <string>", "Token address")
    .option("-ua, --uiAmount <number>", "Number USDT amount will be swap to burn token per period")
    .option("-p, --period <number>", "Interval between burns, in seconds")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, token, uiAmount, period } = params;
    console.log("Initialize Burn");
    console.log({ params });
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const ctx = index_1.Context.withProvider(provider, program.programId);
    const pda = new index_1.PDA(programId);
    const burnState = pda.burn(new web3_js_1.PublicKey(token));
    const platform = pda.platform();
    const platformData = yield ctx.fetcher.getPlatform(platform.key, true);
    if (!platformData) {
        throw new Error(`Platform not found`);
    }
    const purchaseMintAccount = platformData.purchaseMintAccount;
    const purchaseMintAccountInfo = yield ctx.connection.getParsedAccountInfo(purchaseMintAccount);
    if (purchaseMintAccountInfo.value == null) {
        throw new Error(`Info of ${purchaseMintAccount} not found`);
    }
    const purchaseDecimals = purchaseMintAccountInfo.value.data.parsed.info
        .decimals;
    const amountPerPeriod = parseFloat(uiAmount) * Math.pow(10, purchaseDecimals);
    if (!(0, lodash_1.isInteger)(amountPerPeriod)) {
        throw new Error(`Amount per period converting token decimals form is not an integer`);
    }
    const signature = yield program.methods
        .initializeBurn(burnState.bump, new spl_token_1.u64(amountPerPeriod), new bn_js_1.BN(period))
        .accounts({
        platformState: platform.key,
        admin: wallet.publicKey,
        burnState: burnState.key,
        mintAccount: new web3_js_1.PublicKey(token),
        systemProgram: web3_js_1.SystemProgram.programId,
        rent: web3_js_1.SYSVAR_RENT_PUBKEY
    })
        .rpc();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("Initialized Burn state for token: ", token);
}));
// yarn cli updateBurnConfig -n mainnet -t {} -ua {} -p {}
program
    .command("updateBurnConfig")
    .description("update burn config")
    .option("-n, --network <string>", "Network: mainnet, testnet, localnet", "mainnet")
    .option("-t, --token <string>", "Token address")
    .option("-ua, --uiAmount <number>", "Number USDT amount will be swap to burn token per period")
    .option("-p, --period <number>", "Interval between burns, in seconds")
    .action((params) => __awaiter(void 0, void 0, void 0, function* () {
    let { network, token, uiAmount, period } = params;
    console.log("Update Burn Config");
    console.log({ params });
    if (!network || (network !== "mainnet" && network !== "testnet" && network !== "localnet")) {
        console.log("Error: -n, --network is required. [mainnet, testnet, localnet]");
        return;
    }
    let rpc = MAINNET_RPC;
    if (network === "testnet") {
        rpc = TESTNET_RPC;
    }
    if (network === "localnet") {
        rpc = LOCAL_RPC;
    }
    if (!process.env.PROGRAM_ID) {
        console.log("PROGRAM_ID is not found in .env file");
        return;
    }
    const program = yield connectCluster(rpc, ownerKey, new web3_js_1.PublicKey(process.env.PROGRAM_ID));
    const { provider, programId } = program;
    const wallet = (0, index_1.keypairFromJson)(ownerKey);
    if (!wallet.publicKey) {
        console.log("Error: Please provide owner key. `export OWNER_KEY=`");
        return;
    }
    console.log(`RPC: ${rpc}`);
    console.log(`Owner: ${wallet.publicKey.toBase58()}`);
    console.log(`Propeasy Program Id: ${programId.toBase58()}`);
    console.log();
    const ctx = index_1.Context.withProvider(provider, program.programId);
    const pda = new index_1.PDA(programId);
    const platform = pda.platform();
    const burnState = pda.burn(new web3_js_1.PublicKey(token));
    const platformData = yield ctx.fetcher.getPlatform(platform.key, true);
    if (!platformData) {
        throw new Error(`Platform not found`);
    }
    const burnData = yield ctx.fetcher.getBurn(burnState.key, true);
    if (!burnData) {
        throw new Error(`Burn not found`);
    }
    const purchaseMintAccount = platformData.purchaseMintAccount;
    const purchaseMintAccountInfo = yield ctx.connection.getParsedAccountInfo(purchaseMintAccount);
    if (purchaseMintAccountInfo.value == null) {
        throw new Error(`Info of ${purchaseMintAccount} not found`);
    }
    const purchaseDecimals = purchaseMintAccountInfo.value.data.parsed.info
        .decimals;
    const amountPerPeriod = parseFloat(uiAmount) * Math.pow(10, purchaseDecimals);
    if (!(0, lodash_1.isInteger)(amountPerPeriod)) {
        throw new Error(`Amount per period converting token decimals form is not an integer`);
    }
    const signature = yield program.methods
        .updateBurnConfig(new spl_token_1.u64(amountPerPeriod), new bn_js_1.BN(period))
        .accounts({
        platformState: platform.key,
        burnState: burnState.key,
        admin: wallet.publicKey,
    })
        .rpc();
    console.log(`https://explorer.renec.foundation/tx/${signature}?cluster=${network}`);
    console.log("Update burn config for: ", token);
}));
program.parse();
